
/***************************************************************************
                   ichataccount.cpp  -  core iChat account class
                             -------------------
    begin                : Sat Mr 8 2003
    copyright            : (C) 2003 by Graham Dennis <u3952328@anu.edu.au>
							Based on JabberAccount by Till Gerken <till@tantalo.net>
							Based on JabberProtocol by Daniel Stone <dstone@kde.org>
							and Till Gerken <till@tantalo.net>.
 
			   Kopete (C) 2001-2003 Kopete developers
			   <kopete-devel@kde.org>.
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qstring.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qmap.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qbuffer.h>

#include <kdebug.h>
#include <kgenericfactory.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <kstandarddirs.h>
#include <klineeditdlg.h>
#include <kapplication.h>
#include <kaboutdata.h>
#include <kextendedsocket.h>
#include <kurl.h>
#include <kio/netaccess.h>

#include "../kopete/kopete.h"
#include "kopetecontact.h"
#include "kopetecontactlist.h"
#include "kopetemetacontact.h"
#include "kopetemessagemanager.h"
#include "kopeteaway.h"
#include "kopeteprotocol.h"
#include "kopeteaccount.h"
#include "kopeteplugin.h"
#include "addcontactpage.h"
//#include "dlgichatsendraw.h"
//#include "dlgichatservices.h"
//#include "dlgichatchatjoin.h"
#include "ichataddcontactpage.h"
//#include "ichatgroupchat.h" 
/*
#include "jid.h"
#include "client.h"
#include "stream.h"
#include "tasks.h"
#include "types.h"
#include "vcard.h"
*/

#include "util.h"

#include "ichataccount.h"

#include <sys/utsname.h>
#include <unistd.h>
#include <netdb.h>

iChatAccount::iChatAccount ( iChatProtocol * parent, const QString & accountId, const char *name ) :
		 KopeteAccount ( parent, accountId, name ), _presenceService(&RClient), _ichatService(&RClient),
		 _presenceBrowser(&RClient), _ichatBrowser(&RClient)
{
	kdDebug( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Instantiating new account " << accountId << endl;

	mProtocol = parent;
	iChatListener = NULL;
	_ichatNULL = NULL;
	_presenceNULL = NULL;
	OrderedFields.append(QString::fromLatin1("version=1"));

	/* Create a new iChatContact for this account, to be returned from
	 * myself(). */ 
	// FIXME Create a ServiceInfo with all our details and make myContact from it.
	ResolveList.setAutoDelete(true);
	_presenceServiceName = accountId.section( '.', 0, 0);
	        {
                char hostname[100];
                char *tmp;
                struct hostent *host;
                if(gethostname(hostname, 100))
                {
                        fprintf(stderr, "Error: couldn't resolve my hostname\n");
                        exit(1);
                }
                host=gethostbyname(hostname);
                if(host==NULL)
                {
                        fprintf(stderr, "Error: couldn't lookup host: %s\n", hostname);
                        exit(1);
                }
                tmp=host->h_addr_list[0];
				sprintf(hostname, "%u_%u_%u_%u", (unsigned char)tmp[0],
                         (unsigned char)tmp[1], (unsigned char)tmp[2], (unsigned char)tmp[3]);
				_ichatServiceName = QString::fromLatin1(hostname);
        }
	UpdateCount=1;

	myContact = new iChatContact ( accountId, accountId.section( '.', 0, 0 ), this, 0L );

	awayDialog = new iChatAwayDialog( this );

	initialPresence = protocol() ->iChatKOSOnline;

}

iChatAccount::~iChatAccount ()
{
	disconnect ();

	delete awayDialog;
	delete myContact;

}

KopeteContact *iChatAccount::myself () const
{
	return myContact;
}

KActionMenu *iChatAccount::actionMenu ()
{
	KActionMenu * m_actionMenu = new KActionMenu( accountId(), myself() ->onlineStatus().iconFor( this ), this );

	m_actionMenu->popupMenu() ->insertTitle(
	    myContact->onlineStatus().iconFor( myContact ),
	    i18n( "%2 <%1>" )
#if QT_VERSION < 0x030200
	    .arg( accountId() ).arg( myContact->displayName() ) );
#else
		.arg( accountId(), myContact->displayName() ) );
#endif

	m_actionMenu->insert( new KAction ( mProtocol->iChatKOSOnline.caption(),
	                                    mProtocol->iChatKOSOnline.iconFor( this ), 0, this, SLOT ( slotGoOnline () ), this,
	                                    "actioniChatConnect" ) );

	m_actionMenu->insert( new KAction ( mProtocol->iChatKOSIdle.caption(),
	                                    mProtocol->iChatKOSIdle.iconFor( this ), 0, this, SLOT ( slotGoIdle () ), this,
	                                    "actioniChatIdle" ) );

	m_actionMenu->insert( new KAction ( mProtocol->iChatKOSDND.caption(),
	                                    mProtocol->iChatKOSDND.iconFor( this ), 0, this, SLOT ( slotGoDND () ), this,
	                                    "actioniChatDND" ) );
/*
	m_actionMenu->insert( new KAction ( mProtocol->iChatKOSInvisible.caption(),
	                                    mProtocol->iChatKOSInvisible.iconFor( this ), 0, this, SLOT ( slotGoInvisible () ), this,
	                                    "actioniChatInvisible" ) );
*/
	/*	m_actionMenu->insert( new KAction ( mProtocol->iChatKOSOffline.caption(),
		                                    mProtocol->iChatKOSOffline.iconFor( this ), 0, this, SLOT ( slotGoOffline () ), this,
		                                    "actioniChatDisconnect" ) );
	*/
	KAction* mActionOffline = new KAction ( mProtocol->iChatKOSOffline.caption(),
	                                        mProtocol->iChatKOSOffline.iconFor( this ), 0, this, SLOT ( slotGoOffline () ), this,
	                                        "actioniChatDisconnect" );
	mActionOffline->setEnabled( isConnected() );
	m_actionMenu->insert( mActionOffline );
	/*	m_actionMenu->insert( new KAction ( i18n ( "Edit User Info..." ), "identity", 0,
		                                    this, SLOT ( slotEditVCard () ), this, "actionEditVCard" ) );
	*/ 
	return m_actionMenu;
}

/*
 *  Add a contact to Meta Contact
 */
bool iChatAccount::addContactToMetaContact ( const QString & contactId, const QString & displayName, KopeteMetaContact * metaContact )
{

	/* collect all group names */
	/*	QStringList groupNames;
		QPtrList<KopeteGroup> groupList = metaContact->groups();
		for ( KopeteGroup * group = groupList.first(); group; group = groupList.next() )
			groupNames += group->displayName();
	*/

	iChatContact * jc = new iChatContact( contactId, displayName, this, metaContact );

	return ( jc != 0 );

}

void iChatAccount::errorConnectFirst ()
{
	KMessageBox::error ( qApp->mainWidget (), i18n ( "Please connect first." ), i18n ( "Error" ) );
}


void iChatAccount::connect ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "called" << endl;

	/* Don't do anything if we are already connected. */
	if ( isConnected () )
		return ;
#if 0		// Let's kill all that old jabber code
	/* This is dirty but has to be done: if a previous connection attempt
	 * failed, psi doesn't handle recovering too well. we are not allowed to
	 * call close in the slotConnected() slot since it causes a crash, so we
	 * have to delete the psi backend altogether here for safety if it still
	 * exists. */
	if ( jabberClient ) {
		ichatClient->close ();
		delete jabberClient;

		jabberClient = 0L;
	}

	jabberClient = new Jabber::Client ( this );
#endif
	if ( RClient.Connect() == -1 ) {
		KMessageBox::error ( qApp->mainWidget (),
		                     i18n ( "Cannot connect to zmDNS daemon. It is likely that you haven't started it. Or don't have it. Try http://zmdns.sf.net" ), i18n ( "mDNS Connect Error" ) );
		return ;
	}

	Port = 5297;
	int ret;
	iChatListener = new KExtendedSocket;
	iChatListener->setBlockingMode(false);
	iChatListener->setSocketFlags(KExtendedSocket::inetSocket | KExtendedSocket::passiveSocket | KExtendedSocket::streamSocket);
	iChatListener->setAddressReusable(true);
	while(Port < 6000) {
		++Port;
		if (!iChatListener->setPort(Port)) {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Failed to set port: " << Port << endl;
			continue;
		}
		ret = iChatListener->listen();
		if(ret==0) {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Listening successfully on host " << iChatListener->bindHost() << endl;
			break;
		} else if(ret==-1) {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Failed listening: errno: " << errno << endl;
			perror("listen");
			iChatListener->reset();
			continue;
		} else if(ret==-2) {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Socket not passive socket: flags: " << iChatListener->socketFlags() << endl;
			Port=6000;
			break;
		} else
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Umm... this is the default case and we have a problem here" << endl;
	}

	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Listening on port: " << Port << endl;
	if(Port == 6000) {
		delete iChatListener;
		iChatListener=NULL;
	} else {
		QObject::connect(iChatListener, SIGNAL(readyAccept()), this, SLOT(slotConnectionReady()));
	}
//	TXTFields["version"] = "1";
	TXTFields["port.p2pj"] = QString::number(Port);
	TXTFields["1st"] = pluginData(protocol(), "FirstName");
	if(!pluginData(protocol(), "LastName").isEmpty())
		TXTFields["last"] = pluginData(protocol(), "LastName");
	if(!pluginData(protocol(), "Email").isEmpty())
		TXTFields["email"] = pluginData(protocol(), "Email");
	if(!pluginData(protocol(), "AIM").isEmpty())
		TXTFields["AIM"] = pluginData(protocol(), "AIM");
	if(!pluginData(protocol(), "BuddyImageURL").isEmpty()) {
		KURL url(pluginData(protocol(), "BuddyImageURL"));
		QByteArray ba;
		getImage(url, &ba);
		setImage(ba);
	}
	
	TXTFields["status"] = "avail";
	_presenceBrowser.Start(QString::fromLatin1( "_presence._tcp" ));
	_ichatBrowser.Start(QString::fromLatin1("_ichat._tcp" ));
	_presenceService.Start(_presenceServiceName.utf8(), QString::fromLatin1( "_presence._tcp"), Port, OrderedFields, TXTFields);
	UpdateCount=1;
	ConvertTXT2iChat(_ichatTXT, TXTFields);
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Advertising on port: " << Port << endl;
	_ichatService.Start(_ichatServiceName.utf8(), QString::fromLatin1("_ichat._tcp"), Port, _ichatTXT);
	QObject::connect( &_presenceBrowser, SIGNAL( sigBrowseResult( RendezvousClient::BrowseInfo ) ),
			this, SLOT( slotBrowseResult( RendezvousClient::BrowseInfo ) ) );
	QObject::connect( &_ichatBrowser, SIGNAL( sigBrowseResult( RendezvousClient::BrowseInfo ) ),
			this, SLOT( slotBrowseResult( RendezvousClient::BrowseInfo ) ) );
	QObject::connect( &RClient, SIGNAL( sigDisconnected() ), this, SLOT( slotConnectionClosed() ) );

	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Setting Presence." << endl;
	setPresence( protocol() ->iChatKOSConnecting, QString::null );	// FIXME: Find a better way to do this. OR do it properly
	setPresence ( initialPresence, myContact->reason () );

}

void iChatAccount::slotConfigChanged()
{
	TXTFields["1st"] = pluginData(protocol(), "FirstName");
	if(!pluginData(protocol(), "LastName").isEmpty())
		TXTFields["last"] = pluginData(protocol(), "LastName");
	else
		TXTFields.remove("last");
	if(!pluginData(protocol(), "Email").isEmpty())
		TXTFields["email"] = pluginData(protocol(), "Email");
	else
		TXTFields.remove("email");
	if(!pluginData(protocol(), "AIM").isEmpty())
		TXTFields["AIM"] = pluginData(protocol(), "AIM");
	else
		TXTFields.remove("AIM");
	UpdateTXT();
}

void iChatAccount::UpdateTXT()
{
	if(_presenceService.isRunning()==true)
		_presenceService.UpdateTXTRecord( OrderedFields, TXTFields );
	if(_ichatService.isRunning()==true) {
		ConvertTXT2iChat(_ichatTXT, TXTFields);
		_ichatService.UpdateTXTRecord(  _ichatTXT);
	}
}

// length of string resulting from encoding
#define b64_encode_len(x) (4 * (((x) + 2) / 3))
static void b64_encode (unsigned char *str, int len, unsigned char *out)
{
    int i;
    unsigned char *cur = out;
    static char table[64] = {
        'A','B','C','D','E','F','G','H',
        'I','J','K','L','M','N','O','P',
        'Q','R','S','T','U','V','W','X',
        'Y','Z','a','b','c','d','e','f',
        'g','h','i','j','k','l','m','n',
        'o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3',
        '4','5','6','7','8','9','+','/'
    };

    for (i = 0; i < len; i += 3)
    {
        *cur++ = table[str[0] >> 2];
        *cur++ = table[((str[0] & 3) << 4) + (str[1] >> 4)];
        *cur++ = table[((str[1] & 0xf) << 2) + (str[2] >> 6)];
        *cur++ = table[str[2] & 0x3f];
        str += 3;
    }

    if (i == len + 1)
        *(cur - 1) = '=';
    else if (i == len + 2)
        *(cur - 1) = *(cur - 2) = '=';
//    *cur = '\0';
}

const char * const iChatAccount::plistHead = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n<plist version=\"1.0\">\n<data>\n";
const char* const iChatAccount::plistTail = "\n</data>\n</plist>\n";


void iChatAccount::ConvertTXT2iChat(QString& TXTString, QMap<QString, QString>& TXTMap)
{
	MemorySeg Raw;	// FIXME: this won't work with phsh fields
	uint16_t Len;
	Raw.AddDefault("subn", 4);
	Raw.AddDefault(htonl(1));	// Version
	Raw.AddDefault(htonl(UpdateCount++));
	Raw.AddDefault(htonl(0));
	Raw.AddDefault(htonl(TXTMap.size()+1)); // Number of fields +1 for slumming=1
	QMap<QString, QString>::iterator it;
	QCString TempBuf;
	Raw.AddDefault(htons(strlen("slumming")));
	Raw.AddDefault("slumming", strlen("slumming"));
	Raw.AddDefault(htons(strlen("1")));
	Raw.AddDefault("1", strlen("1"));
	for(it = TXTMap.begin(); it != TXTMap.end(); ++it)
	{
		TempBuf = it.key().utf8();
		Len = TempBuf.length();
		Raw.AddDefault(htons(Len));
		Raw.AddDefault((const char*)TempBuf, Len);
		TempBuf = it.data().utf8();
		Len = TempBuf.length();
		Raw.AddDefault(htons(Len));
		Raw.AddDefault((const char*)TempBuf, Len);
	}
	MemorySeg Output;
	Output.AddDefault(plistHead, strlen(plistHead));
	unsigned char *b64encoded = (unsigned char*)Output.Alloc(b64_encode_len(Raw.Size()));
	b64_encode((unsigned char*)Raw.BasePtr(), Raw.Size(), b64encoded);
	Output.AddDefault(plistTail, strlen(plistTail));
	TXTString = QString::fromLatin1((char*)Output.BasePtr(), Output.Size());
}

void iChatAccount::setImage( QByteArray& _ba)
{
	if(_ba.size() == 0) {
		TXTFields.remove(QString::fromLatin1("phsh"));
		delete _presenceNULL;
		delete _ichatNULL;
		_ichatNULL = _presenceNULL = NULL;
		return;
	}
	MemorySeg Output;
	Output.AddDefault(plistHead, strlen(plistHead));
	unsigned char *b64encoded = (unsigned char*)Output.Alloc(b64_encode_len(_ba.size()));
	b64_encode((unsigned char*)_ba.data(), _ba.size(), b64encoded);
	Output.AddDefault(plistTail, strlen(plistTail));
	Output.AddDefault( '\0');
	
	if(!_presenceNULL) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Creating new _presence NULL record" << endl;
		_presenceNULL = _presenceService.newExtraRecord(RendezvousClient::DNSType::DNSType_NULL);
	}
	if(!_ichatNULL) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Creating new _ichat NULL record" << endl;
		_ichatNULL = _ichatService.newExtraRecord(RendezvousClient::DNSType::DNSType_NULL);
	}
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Setting data" << endl;

	_presenceNULL->setData(Output.BasePtr(), Output.Size());
	_ichatNULL->setData(Output.BasePtr(), Output.Size());
	TXTFields[QString::fromLatin1("phsh")] = SHA1::digest(_ba);
//	UpdateTXT();
	return;
}


void iChatAccount::slotBrowseResult( RendezvousClient::BrowseInfo bi )
{
	if  (  bi.Active ) {
		RendezvousClient::Resolve *temp = new RendezvousClient::Resolve(bi.Browser->getClient());
		ResolveList.append(temp);
		QObject::connect( temp, SIGNAL(sigServiceResolved( RendezvousClient::ServiceInfo )),
									this, SLOT(slotServiceResolved( RendezvousClient::ServiceInfo)));
		temp->Start(bi.FQDN);
	} else {
		KopeteMetaContact * mc = KopeteContactList::contactList () ->findContact ( protocol () ->pluginId (), accountId (), bi.FQDN.lower() );

		if ( mc ) {
			/*
			 * A metacontact exists that does contain a contact with the same ID
			 */
			iChatContact * jc = ( iChatContact * ) mc->findContact ( protocol () ->pluginId (),
			                     accountId (),
			                     bi.FQDN.lower() );

			if ( jc ) {
				/*
				 * Since the subcontact exists already, we don't recreate it but
				 * merely update its data according to our parameters.
				 */
				kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Contact " << bi.FQDN << " already exists, updating" << endl;
				jc->Resolver()->Stop();
				ResolveList.remove(jc->Resolver());
				jc->mResolve = NULL;
				jc->setIdleTime( 0 );
				jc->slotUpdatePresence( static_cast<iChatProtocol *>( protocol() ) ->iChatKOSOffline, QString::null );
				emit jc->idleStateChanged(jc);
			}
		} else {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "it seems I can't find contact \"" << bi.FQDN.lower() << "\"" << endl;
		}
	}
}

void iChatAccount::slotServiceResolved( RendezvousClient::ServiceInfo si)
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "SlotServiceResolved" << endl;
	KopeteMetaContact* mc =  NULL;
		/*
		 * If no metacontact has been given, try to locate an existing one
		 * that contains a contact with the same ID that we are to create.
		 */
		mc = KopeteContactList::contactList () ->findContact ( protocol () ->pluginId (), accountId (), si.FQDN.lower() );

		if ( mc ) {
			/*
			 * A metacontact exists that does contain a contact with the same ID
			 */
			iChatContact * jc = ( iChatContact * ) mc->findContact ( protocol () ->pluginId (),
			                     accountId (),
			                     si.FQDN.lower() );

			if ( jc ) {
				/*
				 * Since the subcontact exists already, we don't recreate it but
				 * merely update its data according to our parameters.
				 */
				kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Contact " << si.FQDN << " already exists, updating" << endl;
				jc->slotUpdateContact( si );
				return ;
			} else {
				/*
				 * If this code is reached, something is severely broken:
				 * A subcontact of a metacontact exists but we are unable to
				 * retrieve it's pointer.
				 */
				kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "****Warning**** : " << si.FQDN << " already exists, and cannot be found" << endl;
			}
		}


	/*
	 * If we got here and mc is still NULL, the contact is not
	 * in the contact list yet and we need to create a new metacontact
	 * for it.
	 */
 	QMap<QString, QString> TXTMap;

	{
		if(si.FQDN.contains("_presence._tcp"))
			RendezvousClient::Action::ConvertTXT2Map(si.TXTFields, TXTMap);
		else if (si.FQDN.contains("_ichat._tcp"))
			iChatContact::DecodeiChatTXTFields(si.TXTFields, TXTMap);
		else {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Unknown service in: " << si.FQDN << endl;
			return;
		}
		if(TXTMap.contains(QString::fromLatin1("slumming"))) {
			if(TXTMap[QString::fromLatin1("slumming")] == QString::fromLatin1("1")) {
				// The contact is slumming it, so they would be duplicated if we added them
				si.Resolver->Stop();
				ResolveList.remove(si.Resolver);
				return;
			}
		}
	}


	bool isContactInList = true;
	if ( !mc ) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Adding contact " << si.FQDN << " ..." << endl;

		isContactInList = false;

		mc = new KopeteMetaContact ();
		QString temp = QString::fromLatin1( "Rendezvous" );
		KopeteGroup *group = KopeteContactList::contactList() ->getGroup( temp, KopeteGroup::Normal );
		mc->addToGroup( group );

	}

	/*
	 * At this point, we either found the metacontact or created a new
	 * one. The only thing left to do is to create a new iChat contact
	 * inside it.
	 */
	iChatContact *jc = new iChatContact ( si.FQDN.lower(), si, this, mc );
	( void ) jc;
	//	jc->slotUpdateContact(si);
	if ( !isContactInList )
		KopeteContactList::contactList () ->addMetaContact ( mc );
}

void iChatAccount::removeResolver( RendezvousClient::Resolve *resolver)
{
	if(!resolver) return;
	resolver->Stop();
	ResolveList.remove(resolver);
}

/*
void iChatAccount::slotPsiDebug ( const QString & msg )
{
	kdDebug ( ICHAT_DEBUG_PROTOCOL ) << k_funcinfo << "Psi: " << msg << endl;
}*/

void iChatAccount::slotSocketError( int err )
{
	( void ) err;
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "Error received from backend" << endl;
}

void iChatAccount::disconnect ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] disconnect() called" << endl;
	if ( isConnected () ) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Still connected, closing connection..." << endl;
		/* Tell backend class to disconnect. */
		QObject::disconnect( &RClient, SIGNAL( sigDisconnected() ), this, SLOT( slotConnectionClosed() ) );
		ResolveList.clear();
		RClient.StopAll();
		QObject::disconnect( &_presenceBrowser, SIGNAL( sigBrowseResult( RendezvousClient::BrowseInfo ) ), this,
		 SLOT( slotBrowseResult( RendezvousClient::BrowseInfo ) ) );
		 QObject::disconnect(&_ichatBrowser, SIGNAL( sigBrowseResult( RendezvousClient::BrowseInfo ) ), this,
		 	SLOT( slotBrowseResult( RendezvousClient::BrowseInfo ) ) );
/*	QObject::disconnect( &RClient, SIGNAL( sigServiceResolved( Rendezvous::ServiceInfo ) ),
	                  this, SLOT( slotServiceResolved( Rendezvous::ServiceInfo ) ) );*/
	}
	if(iChatListener) {
		iChatListener->reset();
		QObject::disconnect( iChatListener, SIGNAL(readyAccept()), this, SLOT(slotConnectionReady()));
		delete iChatListener;
		iChatListener=NULL;
	}
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Disconnected." << endl;
	slotDisconnected();
}

void iChatAccount::slotConnectionReady()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] We have received a new connection." << endl;
	KExtendedSocket *newConnection = new KExtendedSocket;
	KSocketAddress *peer;
	QPtrList <KAddressInfo> hostAddressList;
	QString host, port;

	if(iChatListener->accept(newConnection)!=0) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Accept() failed!." << endl;
		return;
	}

	peer = const_cast<KSocketAddress*>(newConnection->peerAddress());
	if(peer==NULL) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] peerAddress() failed!." << endl;
		return;
	}
	int sock = newConnection->fd();
	if(KExtendedSocket::resolve(peer, host, port)!=0) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Resolve() failed!." << endl;
		delete peer;
		return;
	}
	newConnection->release();
	delete newConnection;
	peer=NULL;
	newConnection=NULL;
	int error=0;

	hostAddressList = KExtendedSocket::lookup(host, port, 0, &error);
//	kdDebug ( ICHAT_DEBUG_GLOBAL ) <<
//				"[iChatAccount] lookup: " << KExtendedSocket::strError( KExtendedSocket::IO_LookupError, error) << endl;
	hostAddressList.setAutoDelete(true);
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Host: " << host << " Port: " << port << endl;
	for (QPtrListIterator<KAddressInfo> it (hostAddressList); it.current(); ++it) {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Address: " << (*it)->address()->nodeName() << endl;
	}
	iChatContact* jc;
	int count=0;
	QString connectedhost;
	for ( QDictIterator < KopeteContact > it ( contacts () ); it.current (); ++it ) {
		jc = static_cast<iChatContact*>(*it);
		for (QPtrListIterator<KAddressInfo> it (hostAddressList); it.current(); ++it) {
			if(jc->v4Addr) {
				if((*it)->address()->isCoreEqual(jc->v4Addr))
					connectedhost = jc->v4Addr->nodeName();
			}
			if(jc->v6Addr) {
				if((*it)->address()->isCoreEqual(jc->v6Addr))
					connectedhost = jc->v6Addr->nodeName();
			}
		}
		if(!connectedhost.isEmpty()) {
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Connection apparently from: " << jc->userId() << endl;
			jc->slotIncomingConversation(sock, connectedhost, port.toUInt());
			++count;
			break;
		}
	}
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] End Contact search. " << count << " matches" << endl;
	if(count==0) close(sock);
/*	newConnection->reset();
	delete newConnection;*/
}


void iChatAccount::slotConnectionClosed()
{
	if ( isConnected() ) {
		KMessageBox::error ( qApp->mainWidget (),
		                     i18n ( "The zmDNS daemon disconnected us. It is likely that it needs to be restarted." ), i18n ( "Connection to mDNSd lost" ) );
		disconnect();
		return ;
	} else {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "We just recieved a ConnectionClosed notification, but we are disconnected. All is good." << endl;
	}
}

void iChatAccount::slotConnect ()
{
	connect ();
}

void iChatAccount::slotDisconnect ()
{
	disconnect ();
}

void iChatAccount::slotDisconnected ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Disconnected from iChat server." << endl;

	/* It seems that we don't get offline notifications when going offline
	 * with the protocol, so update all contacts manually. */
	for ( QDictIterator < KopeteContact > it ( contacts () ); it.current (); ++it )
		static_cast < iChatContact * >( *it ) ->slotUpdatePresence ( protocol() ->iChatKOSOffline, "disconnected" );
}

/* Set presence (usually called by dialog widget). */
void iChatAccount::setPresence ( const KopeteOnlineStatus & status, const QString & reason )
{
	/*
	 * If we are in the process of connecting, only update our local presence
	 * and don't send anything across the wire.
	 */
	if ( status == protocol() ->iChatKOSConnecting ) {
		kdDebug( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Setting new presence locally (-> connecting)." << endl;

		myContact->slotUpdatePresence ( status, reason );
	} else {
		/*
		 * If we are already connected and changing our presence or if we are connecting
		 * and set our initial presence, send new presence packet to the server.
		 * Sorry for the ugly if() below but the requirement for certain timings to send out presence
		 * packets and the silly implementation of KopeteAccount::isOnline() leave no other choice.
		 */
		if ( isConnected() ) {
			kdDebug( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Sending new presence to the server." << endl;
			if(status == protocol()->iChatKOSOnline) {
				TXTFields["status"] = "avail";
				TXTFields.remove("away");
			} else if(status == protocol()->iChatKOSIdle) {
				TXTFields["status"] = "away";		// Don't laugh, this is correct
#define GMT1_1_2001       978307200
#define UNIX2OSX(x)   ((x)-GMT1_1_2001)
				// This is the time we went idle at.
				TXTFields["away"] = QString::number(UNIX2OSX(QDateTime::currentDateTime().toTime_t()));
#undef GMT1_1_2001
#undef UNIX2OSX
			} else if (status == protocol()->iChatKOSDND) {
				TXTFields["status"] = "dnd";
				TXTFields.remove("away");
			} else if (status == protocol()->iChatKOSOffline) {
				if(_presenceService.isRunning()==true) {
				_presenceService.Stop();
				myContact->slotUpdatePresence ( status, reason );
				}
				if(_ichatService.isRunning()==true) {
					_ichatService.Stop();
				}
				disconnect();
				return;
			} else {
				kdDebug( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Unknown status" << endl;
			}
			if(!reason.isEmpty())
				TXTFields["msg"] = reason.utf8();
			else
				TXTFields.remove("msg");
			if(_presenceService.isRunning()==false) {
				_presenceService.Start( _presenceServiceName, QString::fromLatin1( "_presence._tcp"), Port, TXTFields);
			} else {
				_presenceService.UpdateTXTRecord( OrderedFields, TXTFields);
			}
			ConvertTXT2iChat(_ichatTXT, TXTFields);
			if(_ichatService.isRunning()==false) {
				_ichatService.Start( _ichatServiceName, QString::fromLatin1("_ichat._tcp"), Port, _ichatTXT);
			} else {
				_ichatService.UpdateTXTRecord( _ichatTXT);
			}
			myContact->slotUpdatePresence ( status, reason );

		} else {
			kdDebug( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "We were not connected, presence update aborted." << endl;
		}
	}

}

bool iChatAccount::getImage( KURL& _url, QByteArray* _ba, QPixmap* _pixmap)
{
	QString imagePath;
	if( KIO::NetAccess::download(_url, imagePath ) )
	{
		QPixmap buddyImage;
		if(buddyImage.load(imagePath)==false) {
			KMessageBox::error( NULL, i18n("Could not read \"%1\" as an image file. Please check the file").arg(_url.path()), i18n("Not an image"));
			KIO::NetAccess::removeTempFile(imagePath);
			return false;
		}
//		cout << "Image is " << _url.path() << endl;
//		buddyImage.resize(64,64);
//		cout << "resize complete" << endl;
		int quality=-1;
		int size;
		bool success;
		{
		QByteArray ba;
		QBuffer buffer( ba );
		buffer.open( IO_WriteOnly );
//		cout << "About to perform initial write" << endl;
		success = buddyImage.save( &buffer, "JPEG", quality ); // writes pixmap into ba in JPEG format
//		cout << "Inital write completed: " << (success? "SUCCESS" : "FAILURE" ) << endl;
//		cout << "Initial size is: " << (size=ba.size()) << endl;
		size = ba.size();
		if(size > 3*1024)
			quality=100;
		else if(_ba)
			*_ba = ba;
		}

		while(size > 3*1024) {
			QByteArray ba;
			QBuffer buffer(ba);
			buffer.open( IO_WriteOnly);
			if(quality==10) {
				KMessageBox::error(NULL, i18n("Unable to reduce the quality of the image sufficiently to reduce the size below 3K"), i18n("Image too large"));
				KIO::NetAccess::removeTempFile(imagePath);
				return false;
			}
			quality -= 10;
//			cout << "Quality is: " << quality << endl;
//			cout << "About to write..." << endl;
			success = buddyImage.save(&buffer, "JPEG", quality);
//			cout << "Write completed: " << (success? "SUCCESS" : "FAILURE" ) << endl;
//			cout << "Size is: " << (size=ba.size()) << endl;
			size = ba.size();
			if(size <= 3*1024) {
				if( _ba)
					*_ba = ba;
			}
		}
//		cout << "Quality is set as: " << quality << " filesize is: " << size << endl;
		if(_pixmap)
			*_pixmap = buddyImage;
//		customImage = buddyImage;
		KIO::NetAccess::removeTempFile(imagePath);
	} else {
		KMessageBox::error(
			NULL,
			i18n("Could not import \"%1\". Check access " \
				"permissions / network connection.")
				.arg(_url.path()),
			i18n("Import Error"));
	}
	return true;
}




void iChatAccount::setAway ( bool away, const QString & reason )
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "Setting away mode: " << away << endl;

	if ( away )
		setPresence ( protocol() ->iChatKOSDND, reason );
	else
		setPresence ( protocol() ->iChatKOSOnline, reason );

}

void iChatAccount::setAvailable ( void )
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Coming back from away mode." << endl;
	slotGoOnline ();
}

void iChatAccount::slotGoOnline ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "called." << endl;

	if ( !isConnected () ) {
		/* We are not connected yet, so connect now. */
		initialPresence = protocol() ->iChatKOSOnline;
		connect ();
	} else {
		setPresence ( protocol() ->iChatKOSOnline, "" );
	}

}

void iChatAccount::slotGoOffline ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "called." << endl;

	disconnect ();
}

void iChatAccount::slotGoIdle ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "called." << endl;

	if ( !isConnected () ) {
		/* We are not connected yet, so connect now. */
		initialPresence = protocol() ->iChatKOSIdle;
		connect ();
	} else {
		awayDialog->show( iChatProtocol::iChatIdle );
	}

}

void iChatAccount::slotGoDND ()
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << k_funcinfo << "called." << endl;

	if ( !isConnected () ) {
		/* We are not connected yet, so connect now. */
		initialPresence = protocol() ->iChatKOSDND;
		connect ();
	} else {
		awayDialog->show( iChatProtocol::iChatDND );
	}

}

/*
iChatContact *iChatAccount::createContact ( const QString & jid, const QString & alias, const QStringList & groups, KopeteMetaContact * metaContact )
{
 
	iChatContact * jc = new iChatContact ( jid, alias, groups, this, metaContact );
 
	return jc;
 
}
*/
#if 0
void JabberAccount::slotContactDeleted ( const Jabber::RosterItem & item )
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Deleting contact " << item.jid ().userHost () << endl;

	if ( !contacts () [ item.jid().userHost().lower() ] ) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatProtocl] WARNING: slotContactDeleted() " << "was asked to delete a non-existing contact." << endl;
		return ;
	}

	iChatContact *jc = static_cast < iChatContact * >( contacts () [ item.jid().userHost().lower() ] );

	/* This will also cause the contact to disappear from the metacontact. */
	delete jc;
}
#endif

#if 0
void JabberAccount::slotContactUpdated ( const Jabber::RosterItem & item )
{
	kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] Status update for " << item.jid ().userHost () << endl;

	/* Sanity check. */
	if ( !contacts () [ item.jid().userHost().lower() ] ) {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[iChatAccount] WARNING: slotContactUpdated() " << "was requested to update a non-existing contact." << endl;
		return ;
	}
	// update the contact data
	static_cast < iChatContact * >( contacts () [ item.jid().userHost().lower() ] ) ->slotUpdateContact ( item );
}
#endif
#if 0
void JabberAccount::slotReceivedMessage ( const Jabber::Message & message )
{
	QString userHost;
	JabberContact *contactFrom;

	userHost = message.from ().userHost ();
	contactFrom = static_cast < JabberContact * >( contacts () [ userHost.lower() ] );

	if ( userHost.isEmpty () ) {
		/* If the sender field is empty, it is a server message.
		 *
		 * When I wrote it, this made sense, but should it be displayed
		 * in a KopeteEmailWindow now? -DS */
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[JabberAccount] New server message for us!" << endl;

		KMessageBox::information ( qApp->mainWidget (), message.body (), i18n ( "Jabber: Server Message" ) );
	} else {
		kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[JabberAccount] New message from '" << userHost << "'" << endl;

		/* See if the contact is actually in our roster. */
		if ( !contactFrom ) {
			/* So, either it's a group chat, or we're not subscribed
			 * to them. Either way. */
			kdDebug ( ICHAT_DEBUG_GLOBAL ) << "[JabberAccount] Message received from an " << "unknown contact, creating temporary contact." << endl;

			KopeteMetaContact *metaContact = new KopeteMetaContact ();

			metaContact->setTemporary ( true );

			contactFrom = createContact ( userHost, userHost, QStringList (), metaContact );

			KopeteContactList::contactList () ->addMetaContact ( metaContact );
		}

		/* Pass the message on to the contact. */
		//		contactFrom->slotReceivedMessage ( message );
	}

}
#endif 
/*
void JabberAccount::slotEditVCard ()
{
	myContact->slotEditVCard ();
}*/

bool iChatAccount::addContact( const QString &contactId, const QString &displayName,
                                KopeteMetaContact *parentContact, const KopeteAccount::AddMode mode, const QString &groupName,
                                bool isTemporary )
{
	return KopeteAccount::addContact( contactId, displayName, parentContact, mode, groupName, isTemporary );
}

/*
void JabberAccount::removeContact ( const Jabber::RosterItem & item )
{
	if ( !isConnected () ) {
		errorConnectFirst ();
		return ;
	}
 
	Jabber::JT_Roster * rosterTask = new Jabber::JT_Roster ( jabberClient->rootTask () );
 
	rosterTask->remove ( item.jid () );
	rosterTask->go ( true );
}*/

/*
const int JabberAccount::port () const
{
 
	return pluginData ( protocol (), "Port" ).toInt ();
 
}*/


#include "ichataccount.moc"
