
/***************************************************************************
                ichataccount.h  -  core iChat account class
                             -------------------
    begin                : Sat Mar 8 2003
    copyright            : (C) 2003 by Graham Dennis <u3952328@anu.edu.au>
							Based on JabberAccount by Till Gerken <till@tantalo.net>
							Based on JabberProtocol by Daniel Stone <dstone@kde.org>
							and Till Gerken <till@tantalo.net>.

			   Kopete (C) 2001-2003 Kopete developers
			   <kopete-devel@kde.org>.
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ICHATACCOUNT_H
#define ICHATACCOUNT_H

#include <qwidget.h>
#include <qptrlist.h>
#include <kaction.h>
#include "kopeteaccount.h"
#include "ichatawaydialog.h"
#include "ichatcontact.h"
#include "ichatprotocol.h"
#include "kopeteonlinestatus.h"
//#include "ui/dlgichatsendraw.h"
//#include "jid.h"
//#include "client.h"
//#include "types.h"
#include "rendezvous.h"

// forward declaration required due to cyclic includes
class iChatAwayDialog;
class KExtendedSocket;
class KURL;
class QPixmap;

/* @author Daniel Stone, Till Gerken */

class iChatAccount:public KopeteAccount
{
	Q_OBJECT

public:
	  iChatAccount (iChatProtocol * parent, const QString & accountID, const char *name = 0L);
	/* Standard null destructor. */
	 ~iChatAccount ();

	/* Returns the contact for this account, must be created in ctor. */
	virtual KopeteContact *myself () const;

	/* Returns the action menu for this account. */
	virtual KActionMenu *actionMenu ();

	virtual void setAway (bool away, const QString & reason = QString::null);

	void setPresence (const KopeteOnlineStatus & status, const QString & reason = 0);

	/* to get the protocol from the account */
	iChatProtocol *protocol () const
	{
		return mProtocol;
	}

	RendezvousClient::Client* getRClient()
	{
		return &RClient;
	}
/*
	QString getFirstName() const
	{
		if(!TXTFields.contains("1st"))
			return QString::null;
		return TXTFields["1st"];
	}

	QString getLastName() const
	{
		if(!TXTFields.contains("last"))
			return QString::null;
		return TXTFields["last"];
	}

	QString getEmail() const
	{
		if(!TXTFields.contains("email"))
			return QString::null;
		return TXTFields["email"];
	}

	QString getAIM() const
	{
		if(!TXTFields.contains("AIM"))
			return QString::null;
		return TXTFields["AIM"];
	}
*/
	void removeResolver(RendezvousClient::Resolve*);

	/* Tells the user to connect first before they can do whatever it is
	 * that they want to do. */
	void errorConnectFirst ();

	static bool getImage(KURL&, QByteArray*, QPixmap* =NULL);
	static const char * const plistHead;
	static const char * const plistTail;

public slots:
	/* Connects to the server. */
	void connect ();

	/* Disconnects from the server. */
	void disconnect ();

	/* Sends a presence packet to a node. */
//	void sendPresenceToNode (const KopeteOnlineStatus & status, const QString & reason);

	void slotConfigChanged();

	/**
	 * Adds a contact to this protocol with the specified details
	 *
	 * @param contactId The unique ID for this protocol
	 * @param displayName The displayname of the contact (may equal userId for some protocols
	 * @param parentContact The metacontact to add this contact to
	 * @param groupName The name of the group to add the contact to
	 * @param isTemporary If this is a temporary contact
	 * @return Pointer to the KopeteContact object which was added
	 */
	bool addContact( const QString &contactId, const QString &displayName = QString::null,
		KopeteMetaContact *parentContact = 0L, const KopeteAccount::AddMode mode = KopeteAccount::DontChangeKABC, 
		const QString &groupName = QString::null, bool isTemporary = false) ;

	void setImage(QByteArray&);

protected:
	/* Create a new contact in the specified metacontact.
	 * You shouldn't call this method yourself; for adding contacts, see @ref addContact().
	 *
	 * @param contactID The unique ID for this protocol.
	 * @param displayName The display name of the contact (may equal @param contactID).
	 * @param parentContact The metacontact to add this contact to
	 */
	virtual bool addContactToMetaContact (const QString & contactID, const QString & displayName, KopeteMetaContact * parentContact);

private:
	/* iChatContact for this account. */
	iChatContact * myContact;

	/* away dialog instance */
	iChatAwayDialog *awayDialog;

	void setAvailable ();
//	void removeContact (const RosterItem &);

	/* Set up our actions for the status menu. */
	void initActions ();

	void ConvertTXT2iChat(QString&, QMap<QString, QString>&);
	void UpdateTXT();
	
	iChatProtocol *mProtocol;

	/* Initial presence to set after connecting. */
	KopeteOnlineStatus initialPresence;

	/* Caches the title ID of the account context menu. */
	int menuTitleId;

	RendezvousClient::Client RClient;
	RendezvousClient::Service _presenceService;
	RendezvousClient::Service _ichatService;
	RendezvousClient::ExtraRecord *_presenceNULL;
	RendezvousClient::ExtraRecord *_ichatNULL;

	RendezvousClient::Browse _presenceBrowser;
	RendezvousClient::Browse _ichatBrowser;
	QPtrList<RendezvousClient::Resolve> ResolveList;
	KExtendedSocket *iChatListener;

	QMap<QString, QString> TXTFields;
	QStringList OrderedFields;
	QString _ichatTXT;
	QString _presenceServiceName;
	QString _ichatServiceName;
	unsigned int Port;
	unsigned long UpdateCount;
	/*
	 * Create a new JabberContact
	 */
/*	JabberContact *createContact (const QString & jid, const QString & alias,
								  const QStringList & groups, KopeteMetaContact * metaContact);
*/
	/* Add new contact to the Kopete contact list; this doesn't actually
	 * affect the Jabber roster, it's just an internal method. */
//	void createAddContact (KopeteMetaContact * mc, const RosterItem & item);

private slots:
		/* Connects to the server. */
	void slotConnect ();

	void slotGoOffline ();

	/* Disconnects from the server. */
	void slotDisconnect ();

	void slotDisconnected ();

	/* Set online mode (presence-wise, and connection-wise). */
	void slotGoOnline ();

	/* Set global presence to "away", no reason. */
	void slotGoIdle ();

	/* Set global presence to "do not disturb", no reason. */
	void slotGoDND ();

	/* Set global presence to "invisible", no reason. */
//	void slotGoInvisible ();
#if 0
	/* A new item was added to our roster, so update our contact list.
	 * If this is a new subscription, make sure we action it. */
//	void slotNewContact (const RosterItem &);

	/* Update a contact's details. */
//	void slotContactUpdated (const RosterItem &);

	/* Someone on our contact list revoked their authorization. */
//	void slotContactDeleted (const RosterItem &);

	/* Displays a new message. */
//	void slotReceivedMessage (const Message &);
#endif
	void slotBrowseResult( RendezvousClient::BrowseInfo);
	void slotServiceResolved( RendezvousClient::ServiceInfo);

	void slotSocketError(int);
	void slotConnectionClosed();

	void slotConnectionReady();


};

#endif
