 /*
  *   ichatprotocol.cpp  -  Base class for the Kopete iChat protocol
  *
  * Copyright (c) 2003 by Graham Dennis <u3952328@anu.edu.au>
  * Copyright (c) 2002-2003 by Till Gerken <till@tantalo.net>
  * Copyright (c) 2002 by Daniel Stone <dstone@kde.org>
  *
  *  Kopete   (c) by the Kopete developers  <kopete-devel@kde.org>
  *
  * *************************************************************************
  * *                                                                       *
  * * This program is free software; you can redistribute it and/or modify  *
  * * it under the terms of the GNU General Public License as published by  *
  * * the Free Software Foundation; either version 2 of the License, or     *
  * * (at your option) any later version.                                   *
  * *                                                                       *
  * *************************************************************************
  */

#include <kdebug.h>
#include <kgenericfactory.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <kstandarddirs.h>
#include <klineeditdlg.h>

#include <qapplication.h>
#include <qcursor.h>
#include <qmap.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qstringlist.h>

//#include "client.h"
#include "stream.h"
//#include "tasks.h"
//#include "types.h"
//#include "vcard.h"

#include <sys/utsname.h>

#include "../kopete/kopete.h"
#include "kopetecontact.h"
#include "kopetecontactlist.h"
#include "kopetemetacontact.h"
#include "kopetemessagemanager.h"
#include "kopeteaway.h"
#include "kopeteprotocol.h"
#include "kopeteplugin.h"
#include "addcontactpage.h"
#include "ichatcontact.h"
#include "ichataddcontactpage.h"
#include "ichatprotocol.h"
#include "ichataccount.h"
#include "kopeteaccountmanager.h"
#include "ichateditaccountwidget.h"
#include "kopetecommandhandler.h"


iChatProtocol *iChatProtocol::protocolInstance = 0;

//K_EXPORT_COMPONENT_FACTORY (kopete_ichat, KGenericFactory < iChatProtocol >);
typedef KGenericFactory<iChatProtocol> iChatProtocolFactory;

K_EXPORT_COMPONENT_FACTORY( kopete_ichat, iChatProtocolFactory( "kopete_ichat" )  )

iChatProtocol::iChatProtocol (QObject * parent, const char *name, const QStringList &) : KopeteProtocol( iChatProtocolFactory::instance(), parent, name ),
	iChatKOSOnline(KopeteOnlineStatus::Online,         25, this, 0, QString::null,       i18n ("O&nline"), i18n ("Online")),
	iChatKOSIdle(KopeteOnlineStatus::Away,             25, this, 2, "ichat_away",       i18n ("&Idle"), i18n ("Idle")),
	iChatKOSDND(KopeteOnlineStatus::Away,              15, this, 4, "ichat_na",         i18n ("&Away"), i18n ("Away")),
	iChatKOSOffline(KopeteOnlineStatus::Offline,       20, this, 5, QString::null,       i18n ("O&ffline"), i18n ("Offline")),
	iChatKOSConnecting(KopeteOnlineStatus::Connecting,  2, this, 7, "ichat_connecting", i18n ("FIXME: You should not see this"), i18n("Connecting"))
{

	kdDebug (ICHAT_DEBUG_GLOBAL) << "[iChatProtocol] Loading ..." << endl;

	/* This is meant to be a singleton, so we will check if we have
	 * been loaded before. */
	if (protocolInstance)
	{
		kdDebug (ICHAT_DEBUG_GLOBAL) << "[iChatProtocol] Warning: Protocol already " << "loaded, not initializing again." << endl;
		return;
	}

	protocolInstance = this;

	// read the iChat ID from Kopete's configuration
	KGlobal::config ()->setGroup ("iChat");

	addAddressBookField ("messaging/ichat", KopetePlugin::MakeIndexField);
}

iChatProtocol::~iChatProtocol ()
{
	//disconnectAll();

	/* make sure that the next attempt to load iChat
	 * re-initializes the protocol class. */
	protocolInstance = 0L;
}



AddContactPage *iChatProtocol::createAddContactWidget (QWidget * parent, KopeteAccount * i)
{
	kdDebug (ICHAT_DEBUG_GLOBAL) << "[iChat Protocol] Create Add Contact  Widget\n" << endl;;
	return new iChatAddContactPage (i, parent);
}

EditAccountWidget *iChatProtocol::createEditAccountWidget (KopeteAccount * account, QWidget * parent)
{
	kdDebug (ICHAT_DEBUG_GLOBAL) << "[iChat Protocol] Edit Account Widget\n" << endl;
	return new iChatEditAccountWidget (this, static_cast < iChatAccount * >(account), parent);
}

KopeteAccount *iChatProtocol::createNewAccount (const QString & accountId)
{
	kdDebug (ICHAT_DEBUG_GLOBAL) << "[iChat Protocol] Create New Account. ID: " << accountId << "\n" << endl;
	return new iChatAccount (this, accountId);
}


iChatProtocol *iChatProtocol::protocol ()
{
	// return current instance
	return protocolInstance;
}

void iChatProtocol::deserializeContact (KopeteMetaContact * metaContact,
										 const QMap < QString, QString > &serializedData, const QMap < QString, QString > & /* addressBookData */ )
{
//  kdDebug (ICHAT_DEBUG_GLOBAL) << k_funcinfo << "Deserializing data for metacontact " << metaContact->displayName () << "\n" << endl;

	QString contactId = serializedData["contactId"];
	QString displayName = serializedData["displayName"];
	QString accountId = serializedData["accountId"];

	QDict < KopeteAccount > accounts = KopeteAccountManager::manager ()->accounts (this);
	KopeteAccount *account = accounts[accountId];

	if (!account)
	{
		kdDebug(ICHAT_DEBUG_GLOBAL) << k_funcinfo << "WARNING: Account for contact does not exist, skipping." << endl;
		return;
	}

	if (account)
		account->addContact (contactId, displayName, metaContact);

}

#include "ichatprotocol.moc"
