 /*
  * ichatprotocol.h  -  Base class for the Kopete iChat protocol
  *
  * Copyright (c) 2003 by Graham Dennis <u3952328@anu.edu.au>
  * Copyright (c) 2002-2003 by Till Gerken <till@tantalo.net>
  * Copyright (c) 2002 by Daniel Stone <dstone@kde.org>
  *
  * Kopete    (c) by the Kopete developers  <kopete-devel@kde.org>
  *
  * *************************************************************************
  * *                                                                       *
  * * This program is free software; you can redistribute it and/or modify  *
  * * it under the terms of the GNU General Public License as published by  *
  * * the Free Software Foundation; either version 2 of the License, or     *
  * * (at your option) any later version.                                   *
  * *                                                                       *
  * *************************************************************************
  */

#ifndef ICHATPROTOCOL_H
#define ICHATPROTOCOL_H

#include <qstring.h>
#include <qstringlist.h>
#include <qmap.h>
#include <qpixmap.h>
#include <qmovie.h>
#include <kaction.h>
#include <kpopupmenu.h>
/*
#include "client.h"
#include "types.h"
#include "jid.h"
*/
#include "kopetecontact.h"
#include "kopetemetacontact.h"
#include "kopeteonlinestatus.h"
#include "addcontactpage.h"

#define ICHAT_DEBUG_GLOBAL		16130
#define ICHAT_DEBUG_PROTOCOL	16131

class iChatContact;
class dlgiChatStatus;

//using namespace Jabber;

class iChatProtocol:public KopeteProtocol
{
	Q_OBJECT

public:

	/**
	 * Object constructor and destructor
	 */
	 iChatProtocol (QObject * parent, const char *name, const QStringList &);
	 ~iChatProtocol ();

	/**
	 * Creates the "add contact" dialog specific to this protocol
	 */
	virtual AddContactPage *createAddContactWidget (QWidget * parent, KopeteAccount * i);
	virtual EditAccountWidget *createEditAccountWidget (KopeteAccount * account, QWidget * parent);
	virtual KopeteAccount *createNewAccount (const QString & accountId);

	/**
	 * Deserialize contact data
	 */
	virtual void deserializeContact (KopeteMetaContact * metaContact,
									 const QMap < QString, QString > &serializedData, const QMap < QString, QString > &addressBookData);

	enum OnlineStatus { iChatOnline, iChatIdle, iChatDND,
						iChatOffline, iChatConnecting };

	const KopeteOnlineStatus iChatKOSOnline;
	const KopeteOnlineStatus iChatKOSIdle;
	const KopeteOnlineStatus iChatKOSDND;
	const KopeteOnlineStatus iChatKOSOffline;
	const KopeteOnlineStatus iChatKOSConnecting;

	/**
	 * This returns our protocol instance
	 */
	static iChatProtocol *protocol ();

	/**
	 * Return whether the protocol supports offline messages.
	 */
	bool canSendOffline() const { return true; }

private:
	/*
	 * Singleton instance of our protocol class
	 */
	static iChatProtocol *protocolInstance;

};

#endif
