/***************************************************************************
*   Copyright (C) 2003 by Graham Dennis                                   *
*   u3952328@anu.edu.au                                                   *
*                                                                         *
*   Permission is hereby granted, free of charge, to any person obtaining *
*   a copy of this software and associated documentation files (the       *
*   "Software"), to deal in the Software without restriction, including   *
*   without limitation the rights to use, copy, modify, merge, publish,   *
*   distribute, sublicense, and/or sell copies of the Software, and to    *
*   permit persons to whom the Software is furnished to do so, subject to *
*   the following conditions:                                             *
*                                                                         *
*   The above copyright notice and this permission notice shall be        *
*   included in all copies or substantial portions of the Software.       *
*                                                                         *
*   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
*   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
*   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
*   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
*   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
*   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
*   OTHER DEALINGS IN THE SOFTWARE.                                       *
***************************************************************************/

#ifndef __PROTOCOL_H
 #define __PROTOCOL_H

#include "ipc.h"
#include <string>
#include <ext/hash_map>

namespace Rendezvous
{

	class IPCProtV1;
	class UDSocketIPC;

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//							IPCProtocol-specific classes									//
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	class IPCProtV1 : public IPCProtocol
				/*  Virtuals to be filled from IPCProtocol:
					virtual MemorySeg&	EncodeMessage(Message&) =0;
					virtual int		DecodeMessages(SocketIPC*) =0;
				*/
	{
		static const char *SyncString;		// String to be at the start of every message
	public:
		IPCProtV1(Instance*);

		MemorySeg &EncodeMessage( Message&, MemorySeg& );
		int DecodeMessages( IPC* );

	};


	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//						Socket-Protocol-specific classes									//
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	class UDSocketIPC : public SocketIPC
				/*  Virtuals to be filled from SocketIPC:
					virtual int Connect() = 0;
				*/
	{
	protected:
		struct sockaddr_un remote;
	public:
		static const char* const SocketPath;

		UDSocketIPC( Instance* i, int newsocket = -1, struct sockaddr_un* newremote = NULL );
		virtual ~UDSocketIPC();

		int Connect();
	};

	class TCPSocketIPC : public SocketIPC
				/*  Virtuals to be filled from SocketIPC:
					virtual int Connect() = 0;
				*/
	{
	protected:
		struct sockaddr_in remote;
		string Host;
		unsigned int Port;
	public:
		static const unsigned int DefaultPort = 5487;
		static const char* const DefaultHost;

		TCPSocketIPC( Instance* i, int newsocket , struct sockaddr_in* newremote );
		TCPSocketIPC( Instance* i, const char*const Host = TCPSocketIPC::DefaultHost,
								unsigned int Port = TCPSocketIPC::DefaultPort );
		virtual ~TCPSocketIPC();

		int Connect();
	};

};


#endif
