/***************************************************************************
*   Copyright (C) 2003 by Graham Dennis                                   *
*   u3952328@anu.edu.au                                                   *
*                                                                         *
*   Permission is hereby granted, free of charge, to any person obtaining *
*   a copy of this software and associated documentation files (the       *
*   "Software"), to deal in the Software without restriction, including   *
*   without limitation the rights to use, copy, modify, merge, publish,   *
*   distribute, sublicense, and/or sell copies of the Software, and to    *
*   permit persons to whom the Software is furnished to do so, subject to *
*   the following conditions:                                             *
*                                                                         *
*   The above copyright notice and this permission notice shall be        *
*   included in all copies or substantial portions of the Software.       *
*                                                                         *
*   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
*   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
*   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
*   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
*   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
*   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
*   OTHER DEALINGS IN THE SOFTWARE.                                       *
***************************************************************************/

#include <iostream>
#include <unistd.h>

#include "rez.h"

using namespace Rendezvous;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//								class MSGProtV1										//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

MSGProtV1::MSGProtV1(Instance* i) : MSGProtocol( 1, i )
{
	MHMap[ "DefaultHandler" ] = MSGProtV1::DefaultHandler;
}

MSGProtV1::~MSGProtV1()
{
	MHMap.erase("DefaultHandler");
}

int	MSGProtV1::DefaultHandler	( Message*msg, IPC* ipc )
{
	(void)ipc;	// Unused parameter
	Message::HashMapIterator hm_it;
	int k = msg->Data.size();

	cout << "Begin unhandled message: " << endl;
	for ( hm_it = msg->Data.begin(); k > 0;k--, hm_it++ ) {
		cout << "Field: \"" << ( char* ) ( *hm_it ).first << "\"\tData (" << ( *hm_it ).second.first << " bytes):";
		if ( ( *hm_it ).second.first < 30 ) cout << " \"";
		else	cout << endl;
		fflush( stdout );
		write( STDOUT_FILENO, ( *hm_it ).second.second, ( *hm_it ).second.first );
		if ( ( *hm_it ).second.first < 30 ) cout << "\"" << endl;
		else cout << endl;
	}

	cout << "End unhandled message" << endl;
	return 0;
}

int MSGProtV1::NullHandler (Message* msg, IPC* ipc)
{
	(void)msg;
	(void)ipc;
	return 0;
}

int MSGProtV1::TranslateNum( Message* msg, IPC* ipc, const char* key, unsigned long& num )
{
	if ( msg->Data.count( key ) == 1 ) {
		char * end;

		num = strtoul( (char*) msg->Data[ key ].second , &end, 10 );
		if ( num == ULONG_MAX ) {
			if ( ipc ) ipc->SendError(msg, "Number exceeded bounds of the unsigned long type");
		} else if ( *end != '\0' ) {
			if ( ipc ) ipc->SendError( msg, "Number had non-parseable characters");
		} else {
			return 0;
		}
	} else {
		if(ipc) ipc->SendError( msg, "ID field does not exist");
	}
	return -1;
}

int MSGProtV1::TranslateNum( Message* msg, IPC* ipc, const char* key, unsigned int& value )
{
	unsigned long id;
	if ( msg->Data.count( key ) == 1 ) {
		char * end;

		id = strtoul( (char*) msg->Data[ key ].second , &end, 10 );
		if ( id == ULONG_MAX ) {
			if ( ipc ) ipc->SendError( msg, "Number exceeded bounds of the unsigned long type" );
		} else if ( *end != '\0' ) {
			if ( ipc ) ipc->SendError( msg, "Number had non-parseable characters" );
		} else if ( id > UINT_MAX ) {
			if ( ipc ) ipc->SendError( msg, "Number exceeded bounds of the unsigned int type" );
		} else {
			value = id;
			return 0;
		}
	} else {
		if(ipc) ipc->SendError( msg, "ID field does not exist");
	}
	return -1;
}


int MSGProtV1::ProcessMessages( IPC* ipc )
{
	Message::MessageListIterator mlist_it = MSGList( ipc ).begin();
	Message *temp;
	//	Message::HashMapIterator hm_it;
	int j = MSGList( ipc ).size();
	//	int k;
	//	cout << j << " messages received:" << endl;
	for ( int i = 0;i < j;i++ ) {
		/*		cout << "Begin Message #" << (i+1) << endl;
				k = ( *mlist_it ) ->Data.size();
				cout << k << " Fields:" << endl;
		 
				for ( hm_it = ( *mlist_it ) ->Data.begin(); k > 0;k--, hm_it++ ) {
					cout << "Field: \"" << (char*) ( *hm_it ).first <<  "\"\tData (" << (*hm_it).second.first << " bytes):";
					if((*hm_it).second.first <30) cout << " \"";
					else		cout << endl;
					fflush(stdout);
					write( STDOUT_FILENO, ( *hm_it ).second.second, ( *hm_it ).second.first );
					if((*hm_it).second.first <30) cout << "\"" << endl;
					else 	cout << endl;
				}
		 
				cout << "End Message #" <<( i+1) << endl;
		*/
		if ( ( *mlist_it ) ->Data.count( "Function" ) == 1 ) {
			char * func = ( char* ) ( *mlist_it ) ->Data[ "Function" ].second;
			if ( MHMap.count( func ) == 1 ) {
#if 1
				if ( MHMap.count( "DefaultHandler" ) == 1 )
					( *MHMap[ "DefaultHandler" ] ) ( *mlist_it, ipc );
#endif
				( *MHMap[ func ] ) ( *mlist_it, ipc );
			} else if ( MHMap.count( "DefaultHandler" ) == 1 ) {
				cout << "No, really. There is no handler to deal with function: " << func << endl;
				( *MHMap[ "DefaultHandler" ] ) ( *mlist_it, ipc );
			} else {
				cout << "No Handler or DefaultHandler installed to deal with function: \"" << func << "\"" << endl;
			}
		} else if ( MHMap.count( "DefaultHandler" ) == 1 ) {
			( *MHMap[ "DefaultHandler" ] ) ( *mlist_it, ipc );
		} else {
			cout << "No DefaultHandler installed to deal with unnamed function" << endl;
		}

		temp = *mlist_it;
		++mlist_it;
		MSGList( ipc ).remove( temp );
		delete temp ;
	}
	return 0;
}

