
/***************************************************************************
                   ichataccountwidget.h  -  Account widget for iChat
                             -------------------
    begin                : Mon Dec 9 2002
    copyright            : (C) 2003 by Graham Dennis <u3952328@anu.edu.au>
                             (C) 2002-2003 by Till Gerken <till@tantalo.net>
                           Based on code by Olivier Goffart <ogoffart@tiscalinet.be>
    email                : kopete-devel@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qbuffer.h>
#include <kmessagebox.h>
#include <klineedit.h>
#include <klocale.h>
#include <kio/netaccess.h>
//#include <kstandarddirs.h>
#include <kurlrequesterdlg.h>


#include "ichateditaccountwidget.h"

/* Remember to change image0 in dlgichateditaccoutwidget.h to protected instead of private
*/






iChatEditAccountWidget::iChatEditAccountWidget (iChatProtocol * proto, iChatAccount * ident, QWidget * parent, const char *name)
						: DlgiChatEditAccountWidget (parent, name), EditAccountWidget (ident)
{
	m_protocol = proto;

	connect (edtFirstName, SIGNAL (textChanged (const QString &)), this, SLOT (configChanged ()));
	connect (edtLastName, SIGNAL (textChanged (const QString &)), this, SLOT (configChanged ()));
	connect (edtEmail, SIGNAL (textChanged (const QString &)), this, SLOT (configChanged ()));
	connect (edtAIM, SIGNAL (textChanged (const QString &)), this, SLOT (configChanged ()));
	connect (mAutoLogon, SIGNAL (toggled (bool)), this, SLOT (configChanged ()));
	connect (mBuddyImage, SIGNAL(clicked()), this, SLOT(buddyImageClicked()));
//	edtServiceName->setDisabled(true);

	if (m_account)
	{
		this->reopen ();
	}
}

iChatEditAccountWidget::~iChatEditAccountWidget ()
{
}


void iChatEditAccountWidget::buddyImageClicked()
{
	KURL chosenImage = KURLRequesterDlg::getURL(buddyImageURL, this,
		i18n("Choose Buddy Image"));
	configChanged();
	kdDebug (ICHAT_DEBUG_GLOBAL) << "buddyImageClicked() called" << endl;
	if(chosenImage.isEmpty()) {
		mBuddyImage->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, mBuddyImage->sizePolicy().hasHeightForWidth() ) );
 		mBuddyImage->setPixmap( image0 );
		buddyImageURL = QString::null;
		imagedata.resize(0);
		return;
	}
	if(iChatAccount::getImage(chosenImage, &imagedata, &customImage)==false) return;
	mBuddyImage->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, mBuddyImage->sizePolicy().hasHeightForWidth() ) );
	mBuddyImage->setPixmap( customImage );
	buddyImageURL = chosenImage.url();
}


void iChatEditAccountWidget::reopen ()
{

	// FIXME: this is temporary until Kopete supports account ID changes!
//	edtServiceName->setDisabled(true);

//	edtServiceName->setText (m_account->accountId ().section('.', 0, 0));
	edtFirstName->setText (m_account->pluginData(m_protocol, "FirstName"));
	edtLastName->setText (m_account->pluginData (m_protocol, "LastName"));

	edtEmail->setText (m_account->pluginData (m_protocol, "Email"));
	edtAIM->setText (m_account->pluginData(m_protocol, "AIM"));
	buddyImageURL = m_account->pluginData(m_protocol, "BuddyImageURL");
	if(!buddyImageURL.isEmpty()) {
		KURL url(buddyImageURL);
		iChatAccount::getImage(url, &imagedata, &customImage);
		mBuddyImage->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, mBuddyImage->sizePolicy().hasHeightForWidth() ) );
		mBuddyImage->setPixmap( customImage );
	}
	mAutoLogon->setChecked (m_account->autoLogin());
}

KopeteAccount *iChatEditAccountWidget::apply ()
{
	kdDebug (ICHAT_DEBUG_GLOBAL) << "iChatEditAccount::apply()" << endl;

	if (!m_account)
	{
		QString AccountID = edtFirstName->text();
		if(!edtLastName->text().isEmpty()) {
			AccountID += " " + edtLastName->text();
		}

		m_account = new iChatAccount (m_protocol, AccountID);
	}

	this->writeConfig ();

	return m_account;
}


void iChatEditAccountWidget::writeConfig ()
{

	// FIXME: The call below represents a flaw in the current Kopete API.
	// Once the API is cleaned up, this will most likely require a change.
	//m_account->setAccountId(mID->text());

	m_account->setPluginData (m_protocol, "FirstName", edtFirstName->text ());
	m_account->setPluginData (m_protocol, "LastName", edtLastName->text ());

	m_account->setPluginData (m_protocol, "Email", edtEmail->text ());
	m_account->setPluginData (m_protocol, "AIM", edtAIM->text ());
	m_account->setPluginData (m_protocol, "BuddyImageURL", buddyImageURL);
	m_account->setAutoLogin(mAutoLogon->isChecked());
	static_cast<iChatAccount*>(m_account)->setImage(imagedata);
	settings_changed = false;
	static_cast<iChatAccount*>(m_account)->slotConfigChanged();
}

bool iChatEditAccountWidget::validateData ()
{
	if(edtFirstName->text().isEmpty())
	{
		KMessageBox::sorry(this, i18n("You have not specified your First name."),
							i18n("No First name"));
		return false;
	}

/*	if(edtServiceName->text().contains('.'))
	{
		KMessageBox::sorry(this, i18n("The Service Name you have chosen is invalid. "
							"Please make sure it does not contain a period '.'"),
							i18n("Invalid Service Name"));
		return false;
	}*/

/*	if(edtServiceName->text().isEmpty())
	{
		// the user didn't specify a server, automatically choose one
		QString temp = edtFirstName->text();
		if(!edtLastName->text().isEmpty())
			temp += ' ' + edtLastName->text();
		temp.remove('.');
		edtServiceName->setText(temp);
	}*/
	return true;
}

void iChatEditAccountWidget::configChanged ()
{
	settings_changed = true;
}
#include "ichateditaccountwidget.moc"

// vim: set noet ts=4 sts=4 sw=4:
