//
//
// C++ Implementation: rendezvous
//
// Description:
//
//
// Author: Graham Dennis <u3952328@anu.edu.au>, (C) 2003
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "rendezvous.h"
#include "rez.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <iostream>
#include <qcstring.h>
#include <typeinfo>

using namespace Rendezvous;
using namespace RendezvousClient;
using namespace std;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//								Action classes											//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Action::TranslateTXT( Message* msg, const char* prefix, QStringList& strList )
{
	unsigned long size = 0;
	unsigned int count, i;
	char key[ 256 ];
	if ( prefix )
		snprintf( key, 256, "%sTXT.Count", prefix );
	else
		snprintf( key, 256, "TXT.Count" );
	if ( msg->Data.count( key ) == 0 ) return -1;
	MSGProtV1::TranslateNum( msg, NULL, key, count );
	size += count;
	for ( i = 0;i < count;i++ ) {
		if ( prefix )
			snprintf( key, 256, "%sTXT[%u]", prefix, i );
		else
			snprintf( key, 256, "TXT[%u]", i );
		if ( msg->Data.count( key ) == 0 ) return -1;
		if ( msg->Data[ key ].first > 255 ) return -1;
		size += msg->Data[ key ].first;
	}
	for ( i = 0;i < count;i++ ) {
		if ( prefix )
			snprintf( key, 256, "%sTXT[%u]", prefix, i );
		else
			snprintf( key, 256, "TXT[%u]", i );
		strList += QString::fromUtf8( ( const char* ) msg->Data[ key ].second, msg->Data[ key ].first );
	}
	return 0;
}

void Action::ConvertTXT2Map( const QStringList& strList, QMap<QString, QString>& Map )
{
	for ( QStringList::ConstIterator it = strList.begin(); it != strList.end(); ++it ) {
		Map[ ( *it ).section( '=', 0, 0 ) ] = ( *it ).section( '=', 1 );
	}
}

void Action::ConvertMap2TXT( const QMap<QString, QString>& Map, QStringList& strList )
{
	for ( QMap<QString, QString>::ConstIterator it = Map.begin(); it != Map.end(); ++it ) {
		strList.append( it.key() + '=' + it.data() );
	}
}

void Action::ConvertString2TXT( const QString& TXTRecord, QStringList& strList)
{
	unsigned int start = 0;
	while ( start < TXTRecord.length() ) {
		strList += TXTRecord.mid( start, 255 );
		start += 255;	// start will likely exceed the length of the string, but only the remaining part of the string is returned
	}
}

int Action::StopAction( const char* s )
{
	if(mID==0) return 0;
	if(stopping == true) return 0;
	int ret = -1;
	stopping = true;
	ret = mClient->Stop( mID, s );
	mClient->DeregisterAction( mID, this);
	mID=0;
	stopping = false;
	return ret;
}




/**********************************************************/

int Query::Start(QString Query, QString QueryType, QString Domain)
{
	Message m;
	m.Function( "StartQuery" );
	m.Add( "Query", Query.utf8() );
	if(!Domain.isEmpty())
		m.Add( "Domain", Domain.utf8() );
	m.Add( "QueryType", QueryType.utf8() );
	return SendMessage( m );
}

int Query::Callback(Message* msg)
{
	if(msg->Data.count("A.Status")==0) return -1;
	if((char*)msg->Data["A.Status"].second == QString::fromLatin1("Active"))
		emit sigQueryResult( this, msg );
	else
		emit sigQueryResultExpire( this, msg);
	return 0;
}

/**********************************************************/

int Browse::Start(QString Type, QString Domain)
{
	Message m;
	m.Function( "StartBrowse" );
	m.Add( "Query", ( const char* ) Type.utf8() );
	if(!Domain.isEmpty())
		m.Add( "Domain", ( const char* ) Domain.utf8() );
	return SendMessage( m );
}

int Browse::Callback(Message* msg)
{
	BrowseInfo bi;
	pair<uint32_t, void*> x;
#define I_AM_SLACK( _a, _b ) if(msg->Data.count(_a)==0) return -1; \
						x = msg->Data[_a]; \
						_b = QString::fromUtf8( (const char*)x.second, x.first-1 );
	I_AM_SLACK( "A.ServiceName", bi.Name );
	I_AM_SLACK( "A.ServiceType", bi.Type );
	I_AM_SLACK( "A.Domain", bi.Domain );
	I_AM_SLACK( "A.FQDN", bi.FQDN );
#undef I_AM_SLACK
	if ( msg->Data.count( "A.Status" ) == 0 ) return -1;
	if ( strcmp( ( const char* ) msg->Data[ "A.Status" ].second, "Active" ) == 0 )
		bi.Active = true;
	else
		bi.Active = false;
/*	if ( MSGProtV1::TranslateNum( msg, NULL, "ID", bi.ID ) == -1 ) {
		bi.ID = 0;
	}*/
	bi.Browser=this;
	emit sigBrowseResult( bi );
	return 0;
}

/**********************************************************/

int NullQuery::Start(QString FQDN)
{
	Message m;
	m.Function( "StartQuery" );
	m.Add( "FQDN", ( const char* ) FQDN.utf8() );
	m.Add( "QueryType", "NULL" );
	return SendMessage( m );
}

int NullQuery::Callback(Message* msg)
{
	QString Data;
	pair<uint32_t, void*> x;
	if(msg->Data.count("A.Data")==0) return -1;
	if(msg->Data.count("A.Status")==0) return -1;
	x = msg->Data["A.Data"];
	Data = QString::fromUtf8( (const char*)x.second, x.first-1);
	if((char*)msg->Data["A.Status"].second == QString::fromLatin1("Active"))
		emit sigNullResult( this, Data );
	else
		emit sigNullExpire( this, Data );
	return 0;
}

/**********************************************************/

int Resolve::Start( QString Name, QString Type, QString Domain )
{
	Message m;
	m.Function( "StartResolve" );
	m.Add( "Name", Name.utf8() );
	m.Add( "Type", Type.utf8() );
	if(!Domain.isEmpty())
		m.Add( "Domain", Domain.utf8() );
	return SendMessage( m );
}

int Resolve::Start( QString FQDN )
{
	Message m;
	m.Function( "StartResolve" );
	m.Add( "FQDN", FQDN.utf8() );
	return SendMessage( m );
}

int Resolve::Callback( Message* msg )
{
	ServiceInfo si;
	if ( msg->Data.count( "A.IPv4" ) == 0 && msg->Data.count( "A.IPv6" ) == 0 ) return 0;
	if ( msg->Data.count( "A.Port" ) == 0 ) return 0;
	if ( msg->Data.count( "A.ServiceName" ) == 0 ) return 0;
	if ( msg->Data.count( "A.ServiceType" ) == 0 ) return 0;
	if ( msg->Data.count( "A.Domain" ) == 0 ) return 0;
	if ( msg->Data.count( "A.FQDN" ) == 0 ) return 0;
	pair<uint32_t, void*> data = msg->Data[ "A.ServiceName" ];
	si.Name = QString::fromUtf8( ( char* ) data.second, data.first - 1 );
	data = msg->Data[ "A.ServiceType" ];
	si.Type = QString::fromUtf8( ( char* ) data.second, data.first - 1 );
	data = msg->Data[ "A.Domain" ];
	si.Domain = QString::fromUtf8( ( char* ) data.second, data.first - 1 );
	if ( msg->Data.count( "A.IPv4" ) == 1 ) {
		data = msg->Data[ "A.IPv4" ];
		si.isAddrV4 = true;
	} else {
		data = msg->Data[ "A.IPv6" ];
		si.isAddrV4 = false;
	}
	si.Address = QString::fromUtf8( ( char* ) data.second, data.first - 1 );
	data = msg->Data[ "A.FQDN" ];
	si.FQDN = QString::fromUtf8( ( char* ) data.second, data.first - 1 );
	MSGProtV1::TranslateNum( msg, NULL, "A.Port", si.Port );
	TranslateTXT( msg, "A.", si.TXTFields );
	si.Resolver=this;
	emit sigServiceResolved( si );
	return 0;
}

/**********************************************************/

ExtraRecord::ExtraRecord(Service* serv, int id, const DNSType type) : _service(serv), RR_ID(id), _type(type) {}

int ExtraRecord::Stop() {
	Message m;
	m.Function( "RemoveRecordFromService" );
	m.Add( "RR_ID", RR_ID);
	return _service->SendMessage(m);
}

int ExtraRecord::Restart() {
	if( _mem.Size() == 0) return 0;
	Message m;
	m.Function("AddRecordToService");
	m.Add( "RR_ID", RR_ID);
	m.Add( "TypeID", _type.Type);
	m.Add( "TTL", _ttl);
	m.AddData( "Data", _mem.BasePtr(), _mem.Size());
	return _service->SendMessage( m );
}

ExtraRecord::~ExtraRecord()
{
	Stop();
}

DNSType ExtraRecord::type() { return _type; }
int ExtraRecord::id() { return RR_ID; }

void ExtraRecord::setData( QByteArray& ba, unsigned int ttl ) {
	setData((void*)(ba.data()), ba.size(), ttl);
}

void ExtraRecord::setData( QCString& cstring, unsigned int ttl)
{
	setData((void*)cstring.data(), cstring.size(), ttl);
}


void ExtraRecord::setData( void* data, size_t len, unsigned int ttl )
{
	_ttl = ttl;
	Message m;
	m.Function( "AddRecordToService" );
	m.Add( "RR_ID", RR_ID);
	m.Add( "TypeID", _type.Type);
	m.Add( "TTL", ttl);
	m.AddData( "Data", data, len);
	if(_service->isRunning())
		_service->SendMessage( m );
	_mem.Free();
	_mem.AddDefault(data, len);
}



/**********************************************************/


int Service::Stop()
{
/*	for(ExtraRecord* extrarr=ExtraRecordList.first(); extrarr; extrarr = ExtraRecordList.next())
		extrarr->Stop();*/
	return StopAction( "DeregisterService" );
}

Service::~Service()
{
	Stop();
}

int Service::Start( QString Name, QString Type, unsigned int Port, QString TXTRecord, QString Domain )
{
	QStringList TXTList;
	ConvertString2TXT( TXTRecord, TXTList);
	return RegisterService( Name, Type, Port, TXTList, Domain );
}

int Service::Start(QString Name, QString Type, unsigned int Port, QMap<QString, QString>& TXTMap, QString Domain )
{
	QStringList TXTList;
	ConvertMap2TXT( TXTMap, TXTList );
	return RegisterService( Name, Type, Port, TXTList, Domain );
}

int Service::Start( QString Name, QString Type, unsigned int Port, QStringList TXTList, QString TXTRecord, QString Domain)
{
	ConvertString2TXT( TXTRecord, TXTList);
	return RegisterService( Name, Type, Port, TXTList, Domain );
}

int Service::Start( QString Name, QString Type, unsigned int Port, QStringList TXTList, QMap<QString,
								 QString>& TXTMap, QString Domain)
{
	ConvertMap2TXT( TXTMap, TXTList );
	return RegisterService( Name, Type, Port, TXTList, Domain );
}


int Service::RegisterService( QString Name, QString Type, unsigned int Port, QStringList& strList, QString Domain )
{
	Message m;
	m.Function( "RegisterService" );
	m.Add( "Name", Name.utf8() );
	m.Add( "Type", Type.utf8() );
	m.Add( "Port", Port );
	if ( !Domain.isEmpty())
		m.Add( "Domain", Domain.utf8() );
	int count = 0;
	QString key( "TXT[%1]" );
	for ( QStringList::ConstIterator it = strList.begin(); it != strList.end(); ++it, count++ ) {
		m.Add( key.arg( count ).latin1(), ( *it ).utf8(), Message::NoNullTerminator );
	}
	m.Add( "TXT.Count", count );

	int ret = SendMessage( m );
	for(ExtraRecord* extrarr=ExtraRecordList.first(); extrarr; extrarr = ExtraRecordList.next())
		extrarr->Restart();
	return ret;
}

int Service::Callback(Message*)
{
	return 0;
}

int Service::UpdateTXTRecord(QStringList& strList, unsigned int ttl)
{
	Message m;
	m.Function("UpdateTXTRecord");
	m.Add("TTL", ttl);
	int count = 0;
	QString key( "TXT[%1]" );
	for ( QStringList::ConstIterator it = strList.begin(); it != strList.end(); ++it, count++ ) {
		m.Add( key.arg( count ).utf8(), ( *it ).utf8(), Message::NoNullTerminator );
	}
	m.Add( "TXT.Count", count );
	SendMessage( m );
	return 0;
}

int Service::UpdateTXTRecord(QString TXTRecord, unsigned int ttl)
{
	QStringList TXTList;
	ConvertString2TXT( TXTRecord, TXTList);
	return UpdateTXTRecord(TXTList, ttl);
}

int Service::UpdateTXTRecord(QMap<QString, QString>& TXTMap, unsigned int ttl)
{
	QStringList TXTList;
	ConvertMap2TXT( TXTMap, TXTList);
	return UpdateTXTRecord(TXTList, ttl);
}

int Service::UpdateTXTRecord( QStringList TXTList, QString TXTRecord, unsigned int ttl )
{
	ConvertString2TXT( TXTRecord, TXTList);
	return UpdateTXTRecord(TXTList, ttl);
}

int Service::UpdateTXTRecord( QStringList TXTList, QMap<QString, QString>& TXTMap, unsigned int ttl )
{
	ConvertMap2TXT( TXTMap, TXTList);
	return UpdateTXTRecord(TXTList, ttl);
}

ExtraRecord* Service::newExtraRecord(DNSType Type)
{
	int i=0;
	ExtraRecord* extra;
	for(extra = ExtraRecordList.first(); extra; extra = ExtraRecordList.next())
	{
		i = max(i, extra->id());
	}
	++i;
//	cout << "Creating extra record, ID: "<< i << endl;
	extra =  new ExtraRecord(this, i, Type);
	ExtraRecordList.append(extra);
	return extra;
}

bool Service::removeExtraRecord(ExtraRecord* extra)
{
	return ExtraRecordList.removeRef(extra);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//								class Client											//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Client::Client() : QObject(), IPC(&__Instance)
{
	Status = CSocket::Disconnected;
	mInstance->AddHandler(1, "Callback",  Client::Callback);
//	cout << "FD_ZERO'ing (1)" << endl;
	FD_ZERO( &IDList );
	AvailID = 1;
}

Client::~Client()
{
	mInstance->RemoveHandler(1, "Callback" );
	Disconnect();
}

void Client::slotConnect() { Connect(); }
void Client::slotDisconnect() { Disconnect(); }

unsigned int Client::getID()
{
	unsigned int ret = AvailID;
	int i;
	FD_SET( AvailID, &IDList);
	for ( i = AvailID + 1; FD_ISSET( i, &IDList ); i++ )
		;
	AvailID = i;
//	cout << "Assigning ID: " << ret << endl;
	return ret;
}

void Client::freeID( unsigned int ID )
{
	if ( FD_ISSET( ID, &IDList ) ) {
		FD_CLR( ID, &IDList );
		if ( ID < AvailID )
			AvailID = ID;
//		cout << "Freeing ID: " << ID <<endl;
	} else
		cout << "Asked to free ID " << ID << " when it isn't registered" << endl;
}

int Client::IDused(unsigned int ID)
{
	return FD_ISSET(ID, &IDList);
}

unsigned int Client::RegisterAction(Action* a)
{
	unsigned int ID = getID();
	ActionsMap[ID] = a;
//	cout << "Registering Action ID " << ID << " to a " << typeid(a).name() << endl;
	return ID;
}

int Client::DeregisterAction(unsigned int ID, Action*)
{
	if(!ActionsMap.contains(ID)) {
		cout << "Something weird is happening here, we were asked to deregister an action we don't have. ID: " << ID << endl;
		return -1;
	}
	int ret = ActionsMap[ID]->Stop();
	ActionsMap.erase(ID);
	return ret;
}

int Client::Callback ( Message*msg, IPC* ipc)
{
	Client* c = dynamic_cast<Client*>(ipc);
	if(c==NULL) {
		cout << "ERROR: we have been passed an IPC* which wasn't a RendezvousClient::Client*" << endl;
		return -1;
	}
	if(msg->Data.count("ID")==0) {
		cout << "No ID field specified in Callback message" << endl;
		return -1;
	}
	unsigned int ID;
	MSGProtV1::TranslateNum(msg, ipc, "ID", ID);
	if(!c->ActionsMap.contains(ID)) {
		cout << "We do not have a reference to ID: " << ID << endl;
		return -1;
	}
	msg->Data.erase("ID");
	msg->Data.erase("Function");
	return c->ActionsMap[ID]->Callback(msg);
}


int Client::Connect()
{
	int len;
	int sock;

	if ( ( sock = socket( AF_UNIX, SOCK_STREAM, 0 ) ) == -1 ) {
		perror( "socket" );
		return -1;
	}
	printf( "Obtained socket: %i\n", sock );

	printf( "Trying to connect...\n" );

	remote.sun_family = AF_UNIX;
	strcpy( remote.sun_path, UDSocketIPC::SocketPath );
	len = strlen( remote.sun_path ) + sizeof( remote.sun_family );
	if ( ::connect( sock, ( struct sockaddr * ) & remote, len ) == -1 ) {
		perror( "connect" );
		return -1;
	}
	printf( "Connected on UD socket %i.\n", sock );
	Status = CSocket::Negotiating;
	sockmgr.setSocket( sock );
	QObject::connect( &sockmgr, SIGNAL( readyRead() ), this, SLOT( slotIncoming() ) );
	QObject::connect( &sockmgr, SIGNAL( error( int ) ), this, SLOT( slotSocketError( int ) ) );
	QObject::connect( &sockmgr, SIGNAL( connectionClosed() ), this, SLOT( slotSocketDisconnected() ) );

	NegotiateProtocol();
	emit sigConnected();
	return 0;
}

int Client::Disconnect()
{
	if ( Status == CSocket::Disconnected ) return 0;
	Buffer.Free();
	setStatus( CSocket::Disconnected );
	QMap<unsigned int, Action*>::iterator it;
	Action* tmpAction;
	for(it=ActionsMap.begin();it!=ActionsMap.end();)
	{
		tmpAction = it.data();
		++it;
		tmpAction->Stop();
	}
	ActionsMap.clear();
	sockmgr.close();
//	cout << "FD_ZERO'ing (2)" << endl;
	FD_ZERO(&IDList);
	AvailID=1;
//	cout << "emitting sigDisconnected();" << endl;
	emit sigDisconnected();
	return 0;
}

int Client::ReceiveAll()
{
	if ( sockmgr.bytesAvailable() == 0 ) return 0;
	Q_ULONG size = sockmgr.bytesAvailable();
	char* buf = ( char* ) Buffer.Alloc( size );
	if ( buf == NULL ) return -1;
	return sockmgr.readBlock( buf, size );
}

CSocket::ConnectionStatus Client::setStatus( CSocket::ConnectionStatus nStatus )
{
	return Status = nStatus;
}

CSocket::ConnectionStatus Client::ConnStatus() { return Status; }

void Client::slotIncoming() { IPCProcessIncoming(); }

int Client::SendMemSeg( MemorySeg& m )
{
	if ( Status == CSocket::Disconnected ) { cout << "Apparently we are disconnected" << endl; return -1; }
	int len = m.Size();
	return sockmgr.writeBlock( ( char* ) m.BasePtr(), len );
}


int Client::Stop( unsigned int ID, const char* Function )
{
	Message m;
	freeID( ID );
	m.Function( Function );
	m.AddTranslate( "ID", ID );
	SendMessage( m );
	return 0;
}

int Client::StopAll()
{
	Message m;
	QMap<unsigned int, Action*>::iterator it;
	Action* tmpAction;
	for(it=ActionsMap.begin();it!=ActionsMap.end();)
	{
		tmpAction = it.data();
		++it;
		tmpAction->Stop();
	}
	ActionsMap.clear();
//	cout << "FD_ZERO'ing (3)" << endl;
	FD_ZERO(&IDList);
	AvailID= 1;
	m.Function( "StopAll" );
	return SendMessage( m );
}

void Client::slotSocketError( int err )
{
	cout << "We have a socket error!" << endl;
	switch ( err ) {
		case QSocket::ErrSocketRead:
			cout << "Error reading on socket" << endl;
			break;
		default:
			cout << "Some other error" << endl;
	}
	emit sigSocketError( err );
}

void Client::slotSocketDisconnected()
{
//	cout << "QSocket disconnected" << endl;
//	cout << ( ( Status == CSocket::Disconnected ) ? "We called the disconnect" : "Someone else called the disconnect" ) << endl;
	if ( Status != CSocket::Disconnected )
		Disconnect();
}

