 /*
  * rendezvousprotocol.h  -  Base class for the Kopete Rendezvous protocol
  *
  * Copyright (c) 2003 by Graham Dennis <u3952328@anu.edu.au>
  * Copyright (c) 2002-2003 by Till Gerken <till@tantalo.net>
  * Copyright (c) 2002 by Daniel Stone <dstone@kde.org>
  *
  * Kopete    (c) by the Kopete developers  <kopete-devel@kde.org>
  *
  * *************************************************************************
  * *                                                                       *
  * * This program is free software; you can redistribute it and/or modify  *
  * * it under the terms of the GNU General Public License as published by  *
  * * the Free Software Foundation; either version 2 of the License, or     *
  * * (at your option) any later version.                                   *
  * *                                                                       *
  * *************************************************************************
  */

#ifndef RENDEZVOUSPROTOCOL_H
#define RENDEZVOUSPROTOCOL_H

#include <qstring.h>
#include <qstringlist.h>
#include <qmap.h>
#include <qpixmap.h>
#include <qmovie.h>
#include <kaction.h>
#include <kpopupmenu.h>
/*
#include "client.h"
#include "types.h"
#include "jid.h"
*/
#include "kopetecontact.h"
#include "kopetemetacontact.h"
#include "kopeteonlinestatus.h"
#include "addcontactpage.h"

#define RENDEZVOUS_DEBUG_GLOBAL		16130
#define RENDEZVOUS_DEBUG_PROTOCOL	16131

class RendezvousContact;
class dlgRendezvousStatus;

//using namespace Jabber;

class RendezvousProtocol:public KopeteProtocol
{
	Q_OBJECT

public:

	/**
	 * Object constructor and destructor
	 */
	 RendezvousProtocol (QObject * parent, const char *name, const QStringList &);
	 ~RendezvousProtocol ();

	/**
	 * Creates the "add contact" dialog specific to this protocol
	 */
	virtual AddContactPage *createAddContactWidget (QWidget * parent, KopeteAccount * i);
	virtual EditAccountWidget *createEditAccountWidget (KopeteAccount * account, QWidget * parent);
	virtual KopeteAccount *createNewAccount (const QString & accountId);

	/**
	 * Deserialize contact data
	 */
	virtual void deserializeContact (KopeteMetaContact * metaContact,
									 const QMap < QString, QString > &serializedData, const QMap < QString, QString > &addressBookData);

	enum OnlineStatus { RendezvousOnline, RendezvousIdle, RendezvousDND,
						RendezvousOffline, RendezvousConnecting };

	const KopeteOnlineStatus RendezvousKOSOnline;
	const KopeteOnlineStatus RendezvousKOSIdle;
	const KopeteOnlineStatus RendezvousKOSDND;
	const KopeteOnlineStatus RendezvousKOSOffline;
	const KopeteOnlineStatus RendezvousKOSConnecting;

	/**
	 * This returns our protocol instance
	 */
	static RendezvousProtocol *protocol ();

	/**
	 * Return whether the protocol supports offline messages.
	 */
	bool canSendOffline() const { return true; }

private:
	/*
	 * Singleton instance of our protocol class
	 */
	static RendezvousProtocol *protocolInstance;

};

#endif
